/** 
 ****************************************************************************
 *
 *
 * @file    TestComponentA.cpp
 *
 * @author  Charles Halliday
 *
 * @date    31 Mar 2008
 *
 ****************************************************************************
 */

#include "TestComponentA.hpp"
#include "rtl/ustring.hxx"

#include <cassert>
#include <unistd.h>

#include <string>

#include "com/sun/star/bridge/XUnoUrlResolver.hpp"

using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::uno;
using namespace ::rtl;
using namespace ::com::myco;
using namespace ::std;


namespace {
  using namespace ::com_myco_TestComponentA;
  using namespace ::com::sun::star::bridge;

  string toStr(OUString ouString)
  {
    string str(OUStringToOString(ouString,
                                 RTL_TEXTENCODING_ASCII_US).getStr());
    return str;
  }  // ! toStr


  OUString toOu(const char *str)
  {
    return OUString::createFromAscii(str);
  }

  Reference < XInterface > getRemoteObject(
    const Reference < XComponentContext > & localContext,
    const OUString & urlObject,
    const sal_Int32 port)
  {
    Reference < XInterface > remoteObject(0);

    Reference < XMultiComponentFactory > localServiceManager(0);

    OUString unoUrl =
      toOu("uno:")
      + toOu("socket,host=localhost")
      + toOu(",port=") + OUString::valueOf(port)
      + toOu(",tcpNoDelay=1")  // as UnoServiceHelper
      + toOu(";")
      + toOu("urp")
      + toOu(",Negotiate=0,ForceSynchronous=0")   // as UnoServiceHelper
      + toOu(";")
      + urlObject;
    DEBUG_LOG("unoUrl=\"" << toStr(unoUrl) << "\"");

    OUString urlResolverServiceName =
      toOu("com.sun.star.bridge.UnoUrlResolver");
    localServiceManager = localContext->getServiceManager();
    assert(localServiceManager.is());

    Reference < XInterface > urlResolverXInterface =
      localServiceManager->createInstanceWithContext(
        urlResolverServiceName, localContext);
    assert(urlResolverXInterface.is());

    Reference < XUnoUrlResolver > urlResolver(urlResolverXInterface,
                                              UNO_QUERY );
    assert(urlResolver.is());

    try {
      remoteObject = urlResolver->resolve(unoUrl);
      DEBUG_LOG("remoteObject.is()="
                << ( remoteObject.is() ? "true" : "false") );
    }
    catch (::com::sun::star::connection::NoConnectException e) {
      ERROR_LOG("NoConnectException in"
                << " urlResolver->resolve() for "
                << toStr(unoUrl));
    }
    catch (::com::sun::star::connection::ConnectionSetupException e) {
      ERROR_LOG("ConnectionSetupException in"
                << " urlResolver->resolve() for "
                << toStr(unoUrl));
    }
    catch (::com::sun::star::lang::IllegalArgumentException e) {
      ERROR_LOG("IllegalArgumentException in"
                << " urlResolver->resolve() for "
                << toStr(unoUrl));
    }
    catch ( ... ) {
      ERROR_LOG("Unknown exception in "
                <<  " urlResolver->resolve() for "
                << toStr(unoUrl) );
    }

    DEBUG_LOG("returning");
    return remoteObject;
  } // ! getRemoteObject

}

namespace com_myco_TestComponentA {

  TestComponentA::TestComponentA(
    Reference< XComponentContext > const & xComponentContext)
    : TestComponentABase::TestComponentABase(xComponentContext)
    , mComponentContext(xComponentContext)
    , mInitialized(sal_False)
  {
  }

  TestComponentA::~TestComponentA()
  {
  }

  // Interface com.sun.star.lang.XInitialization

  // Method of com.sun.star.lang.XInitialization
  void SAL_CALL
  TestComponentA::initialize(
    const Sequence< Any >& aArguments )
    throw (
      Exception,
      RuntimeException )
  {
    ::osl::MutexGuard guard(mInitializedMutex);
    if ( mInitialized == sal_False ) {
      mInitialized = sal_True;
      DEBUG_LOG("called");
    }
  }

  // XMain
  sal_Int32 TestComponentA::run(
    const ::com::sun::star::uno::Sequence< ::rtl::OUString >& aArguments )
    throw (::com::sun::star::uno::RuntimeException)
  {
    DEBUG_LOG("Called");
    int exitValue = 0;

    if ( getArgs(aArguments) ) {

      for ( int i = 0; i < mNumberOfIterations; i++ ) {
#if 0  // No hang if enabled
        usleep(50000); // 50 ms
#endif
        INFO_LOG("Iteration=" << i);
        Reference < XInterface > compAXInterface;  // inside loop
        compAXInterface =
          getRemoteObject(mComponentContext, mUnoObject, mPort);
      
        if ( compAXInterface.is() ) {
          DEBUG_LOG("Cycle " << i << " have remote object");
        }
        else {
          ERROR_LOG("getRemoteObject failed");
          break;
        }
#if 0  // Hang does not seem to occur, if enabled
        compAXInterface.clear();
#endif   
      } // ! for
    // clean up
    }
    else {
      exitValue = 1;
    }

    return exitValue;
  } // ! run

  // Others

  sal_Bool TestComponentA::getArgs(
    const ::com::sun::star::uno::Sequence< ::rtl::OUString >& aArgs )
    throw (::com::sun::star::uno::RuntimeException)
  {
    sal_Bool OK = sal_True;

    mPortAsString = toOu("8100");
    mNumberOfIterations = 1;
    mUnoObject    = toOu("ComponentA");  // As expected by uno executable

    for ( sal_Int32 nextArgNum = 0;
          OK && nextArgNum < aArgs.getLength();
          nextArgNum++) {
      OUString nextArg =  aArgs[nextArgNum];

      if ( nextArg.equalsAscii("-port") ) {
        nextArgNum++;
        if ( nextArgNum < aArgs.getLength() ) {
          mPortAsString = aArgs[nextArgNum];
        }
        else {
          ERROR_LOG("Missing value for -port option");
          OK = sal_False;
        }
      }
      else if ( nextArg.equalsAscii("-n") ) {
        nextArgNum++;
        if ( nextArgNum < aArgs.getLength() ) {
          mNumberOfIterations = aArgs[nextArgNum].toInt32();
        }
        else {
          ERROR_LOG("Missing value for -n option");
          OK = sal_False;
        }
      }
      else if ( nextArg.equalsAscii("-unoobj") ) {
        nextArgNum++;
        if ( nextArgNum < aArgs.getLength() ) {
          mUnoObject = aArgs[nextArgNum];
        }
        else {
          ERROR_LOG("Missing value for -unoobj option");
          OK = sal_False;
        }
      }
      else {
        // Bad/missing option
        ERROR_LOG("Unexpected command line arg:"
                  << toStr(nextArg));
        OK = sal_False;
      }
    }

    mPort = mPortAsString.toInt32();

    DEBUG_LOG("mPort=" << mPort);
    DEBUG_LOG("mNumberOfIterations=" << mNumberOfIterations);

    return OK;
  } // ! getArgs


} // namespace close

